import { Plugin } from 'ckeditor5/src/core';
import { ButtonView } from 'ckeditor5/src/ui';
import type { ElementAttributes as ViewElementAttributes } from '@ckeditor/ckeditor5-engine/src/view/element';
import { attributes, defaultOption } from "./CountDownEditing";
import { CountUpEditorOptions } from "../../../types";
import countDownIcon from '../../../../icons/countdown.svg';
const Drupal = window.Drupal;

export default class CountDownUi extends Plugin {

  init() {
    const editor = this.editor;
    const options = this.editor.config.get('countup_countdown') as CountUpEditorOptions|null;
    if (!options) {
      return;
    }

    const { libraryURL, openDialog, dialogSettings = {} } = options;
    if (!libraryURL || typeof openDialog !== 'function') {
      return;
    }

    editor.ui.componentFactory.add('countup_countdown', (locale) => {
      const buttonView = new ButtonView(locale);

      buttonView.set({
        label: Drupal.t('Insert CountDown'),
        icon: countDownIcon,
        tooltip: true,
      });

      this.listenTo(buttonView, 'execute', () => {
        const selected = this.editor.model.document.selection.getSelectedElement();
        let values: ViewElementAttributes = {};

        if (selected && selected.name === 'countdown-timer') {
          values = Object.entries(attributes).reduce((values, [modelAttribute, viewAttribute]) => {
            let value = selected.getAttribute(modelAttribute) || defaultOption[modelAttribute];
            if (modelAttribute === 'date' && typeof value === 'string') {
              value = value.replaceAll('|', ':');
            }

            values[viewAttribute] = value;
            return values;
          }, {} as Record<string, unknown>);
        }

        openDialog(
          libraryURL,
          (values: any) => {
            editor.model.change(writer => {
              const modelAttributes: ViewElementAttributes = Object.fromEntries(
                Object.entries(attributes).map(
                  ([modelAttribute]) => {
                    let value = (modelAttribute in values && values[modelAttribute] !== null) ? values[modelAttribute] : defaultOption[modelAttribute];
                    if (modelAttribute === 'date' && typeof value === 'string') {
                      value = value.replaceAll(':', '|');
                    }

                    return [modelAttribute, value]
                  }
                )
              );
              const countdown = writer.createElement('countdown-timer', modelAttributes);

              editor.model.insertObject(countdown, null, null, {
                setSelection: 'after'
              });
            });
          },
          dialogSettings,
          values
        );
      });

      return buttonView;
    });
  }

}
