import { Plugin } from 'ckeditor5/src/core';
import { ButtonView } from 'ckeditor5/src/ui';
import type { ElementAttributes as ViewElementAttributes } from '@ckeditor/ckeditor5-engine/src/view/element';
import { attributes, defaultOption } from "./CountUpEditing";
import { CountUpEditorOptions } from "../../../types";
import countUpIcon from '../../../../icons/countup.svg';
const Drupal = window.Drupal;

export default class CountUpUi extends Plugin {

  init() {
    const editor = this.editor;
    const options = this.editor.config.get('countup_countup') as CountUpEditorOptions|null;
    if (!options) {
      return;
    }

    const { libraryURL, openDialog, dialogSettings = {} } = options;
    if (!libraryURL || typeof openDialog !== 'function') {
      return;
    }

    editor.ui.componentFactory.add('countup_countup', (locale) => {
      const buttonView = new ButtonView(locale);

      buttonView.set({
        label: Drupal.t('Insert CountUp'),
        icon: countUpIcon,
        tooltip: true,
      });

      this.listenTo(buttonView, 'execute', () => {
        const selected = this.editor.model.document.selection.getSelectedElement();
        let values: ViewElementAttributes = {};

        if (selected && selected.name === 'countup') {
          values = Object.entries(attributes).reduce((values, [modelAttribute, viewAttribute]) => {
            viewAttribute = viewAttribute.replace('-', '_');
            values[viewAttribute] = selected.getAttribute(modelAttribute) || defaultOption[modelAttribute].value;
            return values;
          }, {} as Record<string, unknown>);
        }

        openDialog(
          libraryURL,
          (values: any) => {
            editor.model.change(writer => {
              const modelAttributes: ViewElementAttributes = Object.fromEntries(
                Object.entries(attributes).map(
                  ([modelAttribute]) => [modelAttribute, (modelAttribute in values && values[modelAttribute] !== null) ? values[modelAttribute] : defaultOption[modelAttribute].value]
                )
              );
              const countup = writer.createElement('countup', modelAttributes);

              editor.model.insertObject(countup, null, null, {
                setSelection: 'after'
              });
            });
          },
          dialogSettings,
          values
        );
      });

      return buttonView;
    });
  }

}
