export default class CountDownTimerDigit extends HTMLElement {
  static get observedAttributes() {
    return ['digit'];
  }

  constructor() {
    super();
    const shadow = this.attachShadow({ mode: 'open' });
    this.element = document.createElement('span');
    this.element.classList.add('digit');
    this.element.setAttribute('part', 'digit');
    this.element.innerHTML =
      '<span class="current"></span><span class="new"></span>';
    const style = document.createElement('style');
    style.textContent = `
      .digit {
        display: grid;
        font-family: var(--cdt-digit-font-family, inherit);
        font-weight: var(--cdt-digit-font-weight, 700);
        color: var(--cdt-digit-text-color, #fefefe);
        background-color: var(--cdt-digit-box-color, #111);
        border-radius: clamp(2px, .1em, 10px);
        position: relative;
      }
      .current, .new {
        padding: .1em;
      }
      .new {
        position: absolute;
        left: 0;
        opacity: 0;
        tranform-origin: bottom center;
        transform: translateY(-40%) scale(.9, .2);
      }
      .flip .current {
        opacity: 0;
        transform-origin: top center;
        transform: translateY(70%) scale(.8, .2);
        transition: .4s;
      }
      .flip .new {
        opacity: 1;
        transform: translateY(0);
        transition: .6s;
      }
    `;
    shadow.appendChild(style);
    shadow.appendChild(this.element);
  }

  attributeChangedCallback(_, oldValue, newValue) {
    const currentElement = this.element.querySelector('.current');
    const newElement = this.element.querySelector('.new');
    if (!oldValue) {
      currentElement.innerText = newValue;
    }
    if (newValue !== oldValue) {
      newElement.innerText = newValue;
      this.element.classList.add('flip');
      setTimeout(() => {
        currentElement.innerText = newValue;
        newElement.innerText = '';
        this.element.classList.remove('flip');
      }, 600);
    }
  }
}
