export default class CountDownTimerNumber extends HTMLElement {
  static get observedAttributes() {
    return ['label', 'number'];
  }

  constructor(){
    super();
    this.digits = [];
  }

  connectedCallback() {
    this.digits = [];
    const shadow = this.attachShadow({ mode: 'open' });
    this.element = document.createElement('div');
    this.element.setAttribute('title', this.getAttribute('label'));
    this.element.insertAdjacentHTML(
      'afterbegin',
      '<div class="wrapper"></div>' +
      `<span part="number-label">${this.getAttribute('label')}</span>`,
    );
    const style = document.createElement('style');
    style.textContent =
      'div {' +
        'display: grid;' +
        'justify-items: center;' +
        'gap: .5rem;' +
      '}' +
      '.wrapper {' +
        'display:flex;' +
        'gap: 2px;' +
      '}' +
      'span {' +
        'font-size: var(--cdt-label-font-size, .75rem);' +
      '}';
    shadow.appendChild(style);
    shadow.appendChild(this.element);
  }

  attributeChangedCallback(name, oldValue, newValue) {
    if (this.digits.length === 0) {
      this.digits = [
        ...this.getAttribute('number')
          .split('')
          .map(() => {
            const digit = document.createElement('ct-digit');
            digit.setAttribute('exportparts', 'digit');
            return digit;
          }),
      ];
      this.digits.forEach((digit) => {
        this.element?.querySelector('.wrapper').appendChild(digit);
      });
    }
    if (this.digits.length > 0) {
      newValue.split('').forEach((value, index) => {
        this.digits[index]?.setAttribute('digit', value);
      });
    }
  }
}
