const path = require('path');
const fs = require('fs');
const webpack = require('webpack');
const TerserPlugin = require('terser-webpack-plugin');

function getDirectories(srcpath) {
  return fs
    .readdirSync(srcpath)
    .filter((item) => fs.statSync(path.join(srcpath, item)).isDirectory());
}

const prodPluginBuilds = [];
const devPluginBuilds = [];

module.exports = [];
// Loop through every subdirectory in src, each a different plugin, and build
// each one in ./build.
getDirectories('./js/ckeditor5_plugins').forEach((dir) => {
  const bc = {
    mode: 'production',
    resolve: {
      extensions: ['.ts', '.js']
    },
    optimization: {
      minimize: true,
      minimizer: [
        new TerserPlugin({
          terserOptions: {
            mangle: {
              reserved: ['Drupal']
            },
            format: {
              comments: false,
            },
          },
          test: /\.(js|ts)$/i,
          extractComments: false,
        }),
      ],
      moduleIds: 'named',
    },
    entry: {
      path: path.resolve(__dirname, 'js/ckeditor5_plugins', dir, 'src/index.ts')
    },
    output: {
      path: path.resolve(__dirname, './js/build'),
      filename: `${dir}.ckeditor5.js`,
      library: ['CKEditor5', dir],
      libraryTarget: 'umd',
      libraryExport: 'default'
    },
    plugins: [
      new webpack.DllReferencePlugin({
        manifest: require('./node_modules/ckeditor5/build/ckeditor5-dll.manifest.json'), // eslint-disable-line global-require, import/no-unresolved
        scope: 'ckeditor5/src',
        name: 'CKEditor5.dll',
      }),
    ],
    module: {
      rules: [
        { test: /\.ts$/, use: 'ts-loader', exclude: /node_modules/ },
        { test: /\.svg$/, use: 'raw-loader' }
      ],
    },
  };

  const dev = {...bc, mode: 'development', optimization: {...bc.optimization, minimize: false}, devtool: false};

  prodPluginBuilds.push(bc);
  devPluginBuilds.push(dev);
});

const bc = {
  mode: 'production',
  resolve: {
    extensions: ['.ts', '.js']
  },
  optimization: {
    sideEffects: true,
    minimize: true,
    minimizer: [
      new TerserPlugin({
        terserOptions: {
          format: {
            comments: false,
          },
        },
        test: /\.(js|ts)$/i,
        extractComments: false,
      }),
    ],
    moduleIds: 'named',
  },
  entry: {
    path: path.resolve(__dirname, 'js/countdown/index.js')
  },
  output: {
    path: path.resolve(__dirname, './js/build'),
    filename: `countdown.js`,
  },
  module: {
    rules: [
      { test: /\.ts$/, use: 'ts-loader', exclude: /node_modules/ },
      { test: /\.svg$/, use: 'raw-loader' }
    ],
  },
};

const dev = {...bc, mode: 'development', optimization: {...bc.optimization, minimize: false}, devtool: false};
prodPluginBuilds.push(bc);
devPluginBuilds.push(dev);

module.exports = (env, argv) => {
  // Files aren't minified in build with the development flag.
  if (argv.mode === 'development') {
    return devPluginBuilds;
  } else {
    return prodPluginBuilds;
  }
}
